/*
 * Copyright (C) 2004 Robert Hogan <robert at roberthogan dot net>
 */

#include <stdlib.h>

#include "freshklam.h"
#include "klamav.h"
#include "klamd.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurlrequesterdlg.h>

#include <kedittoolbar.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qregexp.h>
#include <qwhatsthis.h>
#include <kbuttonbox.h>
#include <kfiledialog.h>
#include <kprocess.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kwin.h>
#include <kurlrequester.h>
#include <kurlcompletion.h>
#include <kcombobox.h>
#include <kdebug.h>
#include <qlayout.h>
#include <kcmdlineargs.h>
#include <kmessagebox.h>
#include <klineedit.h>
#include <ktempfile.h>

const char *check_desc[] = {
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
    "10",
    "11",
    "12",
    "13",
    "14",
    "15",
    "16",
    "17",
    "18",
    "19",
    0
};
Freshklam::Freshklam(QWidget *parent, const char *name)
    : QWidget(parent, name)
{


	config = KGlobal::config();
	config->setGroup("Freshklam");
	lastDownloadPaths = config->readListEntry("lastDownloadPaths");

	if (lastDownloadPaths.isEmpty()){
		lastDownloadPaths.append( "/usr/local/share/clamav");
		config->writeEntry("lastDownloadPaths", lastDownloadPaths);
		lastDownloadPaths = config->readListEntry("lastDownloadPaths");
	}
	QString proxyIPText = config->readEntry("ProxyIP");
	QString proxyPortText = config->readEntry("ProxyPort");
	QString proxyUserText = config->readEntry("ProxyUser");
	QString proxyPassText = config->readEntry("ProxyPass");


	//Data Directory Widget


	QVBoxLayout *vbox = new QVBoxLayout(this, KDialog::marginHint(),
			KDialog::spacingHint(), "vbox");

	QGroupBox *group = new QGroupBox(i18n("Download Directory"), this);
	vbox->addWidget(group);


	QGridLayout *layout = new QGridLayout( group, 9, 2, KDialog::spacingHint(),
			KDialog::spacingHint(), "layout");
	layout->addRowSpacing(0, group->fontMetrics().height());
	layout->setColStretch(0, 1);
	layout->setColStretch(1, 1);


	QWidget *hlp = new QWidget( group );
	layout->addMultiCellWidget(hlp, 1,1, 0,1);
	QHBoxLayout *dir_layout = new QHBoxLayout(hlp, KDialog::spacingHint() );
	dir_combo = new KURLRequester( new KComboBox(true, this), hlp, "dir combo" );
	dir_combo->completionObject()->setMode(KURLCompletion::DirCompletion);
	dir_combo->comboBox()->insertStringList(lastDownloadPaths);
	dir_combo->setMode( KFile::Directory|KFile::LocalOnly );

	QLabel *dir_label = new QLabel(i18n("Directory:"), hlp);
	dir_label->setFixedSize(dir_label->sizeHint());
	dir_layout->addWidget(dir_label);
	dir_layout->addWidget(dir_combo);

	//Proxy Widget
	QGroupBox *proxy_group = new QGroupBox(i18n("Proxy"), this);
	vbox->addWidget(proxy_group);


	QGridLayout *proxy_layout = new QGridLayout( proxy_group, 5, 2, KDialog::spacingHint(),
			KDialog::spacingHint(), "proxy_layout");
	proxy_layout->addRowSpacing(0, proxy_group->fontMetrics().height());
	proxy_layout->setColStretch(0, 1);
	proxy_layout->setColStretch(1, 1);

	//IP Address & Port
	QWidget *proxy_hlp = new QWidget( proxy_group );

	proxy_layout->addMultiCellWidget(proxy_hlp, 1,1, 0,1);
	QHBoxLayout *proxy_dir_layout = new QHBoxLayout(proxy_hlp, KDialog::spacingHint() );

	QLabel *proxy_dir_label = new QLabel(i18n("IP Address:"), proxy_hlp);
	proxy_dir_label->setFixedSize(proxy_dir_label->sizeHint());
	proxy_dir_layout->addWidget(proxy_dir_label);
	proxyIP = new KLineEdit(proxy_hlp);
	proxy_dir_layout->addWidget(proxyIP);
	proxyIP->setText(proxyIPText);
	
	QLabel *proxy_port_label = new QLabel(i18n("Port:"), proxy_hlp);
	proxy_port_label->setFixedSize(proxy_port_label->sizeHint());
	proxy_dir_layout->addWidget(proxy_port_label);
	proxyPort = new KLineEdit(proxy_hlp);
	proxy_dir_layout->addWidget(proxyPort);
	proxyPort->setText(proxyPortText);
	
	//User & Pass
	QWidget *proxy_hlp2 = new QWidget( proxy_group );
	proxy_layout->addMultiCellWidget(proxy_hlp2, 2,2, 0,1);
	QHBoxLayout *proxy_userpass_layout = new QHBoxLayout(proxy_hlp2, KDialog::spacingHint() );

	QLabel *proxy_user_label = new QLabel(i18n("User:"), proxy_hlp2);
	proxy_user_label->setFixedSize(proxy_user_label->sizeHint());
	proxy_userpass_layout->addWidget(proxy_user_label);
	proxyUser = new KLineEdit(proxy_hlp2);
	proxy_userpass_layout->addWidget(proxyUser);
	proxyUser->setText(proxyUserText);
	
	QLabel *proxy_pass_label = new QLabel(i18n("Password:"), proxy_hlp2);
	proxy_pass_label->setFixedSize(proxy_pass_label->sizeHint());
	proxy_userpass_layout->addWidget(proxy_pass_label);
	proxyPass = new KLineEdit(proxy_hlp2);
	proxy_userpass_layout->addWidget(proxyPass);
	proxyPass->setText(proxyPassText);
	
	//Daemon Widget


	QGroupBox *daemon_group = new QGroupBox(i18n("AutoUpdate Settings"), this);
	vbox->addWidget(daemon_group);


	QGridLayout *daemon_layout = new QGridLayout( daemon_group, 5, 2, KDialog::spacingHint(),
			KDialog::spacingHint(), "daemon_layout");
	daemon_layout->addRowSpacing(0, daemon_group->fontMetrics().height());
	daemon_layout->setColStretch(0, 1);
	daemon_layout->setColStretch(1, 1);


	QWidget *daemonhlp = new QWidget( daemon_group );
	daemon_layout->addMultiCellWidget(daemonhlp, 1,1, 0,1);

	QHBoxLayout *daemon_check_layout = new QHBoxLayout(daemonhlp, KDialog::spacingHint() );
	daemon_box = new QCheckBox(i18n("Update Virus Database Automatically"), daemonhlp);
	daemon_box->setMinimumWidth(daemon_box->sizeHint().width());
	connect( daemon_box, SIGNAL(toggled(bool)), SLOT(handleChecks()) );

	daemon_check_layout->addSpacing(10);
	daemon_check_layout->addWidget(daemon_box);

	QWidget *daemonhlp2 = new QWidget( daemon_group );
	daemon_layout->addMultiCellWidget(daemonhlp2, 2,2, 0,1);
	QHBoxLayout *daily_check_layout = new QHBoxLayout(daemonhlp2, KDialog::spacingHint() );

	check_combo = new QComboBox(false, daemonhlp2);
	check_combo->insertStrList(check_desc);
	if (!(config->readEntry("NoOfUpdates").isEmpty()))
		check_combo->setCurrentText(config->readEntry("NoOfUpdates"));
	check_combo->adjustSize();
	check_combo->setFixedSize(check_combo->size());
	daily_check_layout->addWidget(check_combo);
	connect( check_combo, SIGNAL(activated(int)), SLOT(handleChecks()) );

	
	QLabel *combo_label = new QLabel(i18n("Times a Day"), daemonhlp2);
	combo_label->setFixedSize(combo_label->sizeHint());
	daily_check_layout->addWidget(combo_label);

	if ((config->readEntry("AutoUpdate")) == "Yes"){
		daemon_box->setChecked(true);
		check_combo->setEnabled(true);
	}else{
		daemon_box->setChecked(false);
		check_combo->setEnabled(false);
	}


    KButtonBox *actionbox = new KButtonBox(this, Qt::Horizontal);
    vbox->addWidget(actionbox, 2, 0);
    actionbox->addStretch();
    search_button = actionbox->addButton(i18n("&Update Now"));
    search_button->setDefault(true);
    cancel_button = actionbox->addButton(i18n("Cancel"));
    cancel_button->setEnabled(false);
    actionbox->addStretch();
    actionbox->layout();

/*
    resultbox = new QListBox(this);
    QFontMetrics rb_fm(resultbox->fontMetrics());
    resultbox->setMinimumSize(rb_fm.width("0")*55,
			      rb_fm.lineSpacing()*15);
    layout->addMultiCellWidget(resultbox, 2, 2, 0, 2);
*/
    QFrame *status_frame = new QFrame(this);
    status_frame->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    QBoxLayout *status_layout = new QHBoxLayout(status_frame, 2);

    status_label = new QLabel(i18n("Ready"), status_frame);
    status_layout->addWidget(status_label, 10);

/*    matches_label = new QLabel(status_frame);
    QFontMetrics ml_fm(matches_label->fontMetrics());
    matches_label->setFixedWidth(ml_fm.width(i18n("9999 viruseses found")));
    matches_label->setFixedHeight(ml_fm.lineSpacing());
    status_layout->addWidget(matches_label, 0);
*/
    status_layout->activate();
    status_frame->adjustSize();
    status_frame->setMinimumSize(status_frame->size());
     vbox->addWidget(status_frame);


    //vbox->addStretch(1);

    connect( search_button, SIGNAL(clicked()),
	     SLOT(slotSearch()) );
    connect( cancel_button, SIGNAL(clicked()),
	     SLOT(slotCancel()) );

	if ((KApplication::kApplication()->isRestored()) && (daemon_box->isChecked()))
		slotSearch();
	


}

Freshklam::~Freshklam()
{
	
	if (daemon_box->isChecked()){
		killPID();
	}

}

void Freshklam::processOutput()
{
    int pos;
    int pos2;
    QString item2;

    while (( (pos = buf.find('\n')) != -1) || ( (pos = buf.find(']')) != -1))
    {
        QString item = buf.left(pos);
        if (!item.isEmpty()){
		if ( (pos2 = buf.find('[')) != -1)
			item +=']';
           	status_label->setText(item);
		item2 += item;
		item2 += '\n';
	}
	buf = buf.right(buf.length()-pos-1);
    }

	if ((pos = item2.find("ERROR")) != -1){
		//search_button->setEnabled(true);
		//cancel_button->setEnabled(false);
		//delete childproc;
		//childproc = 0;
		//status_label->setText("Ready");
		pos2 = item2.find('\n',pos);
		errorMessage = item2.mid(pos,pos2);
		//KMessageBox::information (this,item2.mid(pos,pos2));
		//finish();
	}







}


void Freshklam::slotSearch()
{

	search_button->setEnabled(false);
	cancel_button->setEnabled(true);
	
	status_label->setText(i18n("Beginning Update..."));
	
	QString filepattern = dir_combo->url();
	
	childproc = new KShellProcess();
	QString command = "freshclam --stdout";
	
	if (!(filepattern.isEmpty())){
	command += " --datadir=";
	command += filepattern;
	}
	
	if (daemon_box->isChecked()){
		command += " -d -c ";
		command += check_combo->currentText();
		KTempFile tf;
		if ( tf.status() != 0 ) {
			tf.close();
			//delete tf;
			KMessageBox::information (this,"There was an error creating a temp file!");
			return;
		}
		pidFileName = tf.name();
		command += " -p ";
		command += pidFileName;
	}
	
	if (!(KApplication::kApplication()->isRestored())){

		if (kmain->klamd->isKlamdAlive()){
			QString klamdconf = kmain->klamd->getKlamdConfFile();
			kdDebug() << klamdconf << endl;
			command += " --daemon-notify=";
			command += klamdconf;
		}		
	}
		
	if (proxyIP->text() != ""){
		writeConf();
		command += " --config-file=";
		command += tempFileName;
	}
	
	disableInputs();
	
	
	*childproc << command;
	
	connect( childproc, SIGNAL(processExited(KProcess *)),
		SLOT(childExited()) );
	connect( childproc, SIGNAL(receivedStdout(KProcess *, char *, int)),
		SLOT(receivedOutput(KProcess *, char *, int)) );
	childproc->start(KProcess::NotifyOnExit, KProcess::Stdout);
	
}

void Freshklam::finish()
{
	if (!(daemon_box->isChecked())){
		search_button->setEnabled(true);
		cancel_button->setEnabled(false);
		enableInputs();
	}
	buf += '\n';
	processOutput();
	delete childproc;
	childproc = 0;
	
}


void Freshklam::slotCancel()
{
	finish();
	search_button->setEnabled(true);
	cancel_button->setEnabled(false);
	
	if (daemon_box->isChecked()){
		if (!(killPID()))
			KMessageBox::information (this,"There was a problem killing the update process!");
		enableInputs();
	}
	

	status_label->setText(i18n("Canceled"));


}


void Freshklam::childExited()
{
    int status = childproc->exitStatus();

    finish();

    if (status == 0){
    	if (daemon_box->isChecked()){
		if (!(KApplication::kApplication()->isRestored()))
			KMessageBox::information (this,"Update Running in Background!");
	}else
		updateMailClient();
    }else if (status == 1)
	KMessageBox::information (this,"No Update Required - Database already up to date!");
    else if (status == 40)
	KMessageBox::information (this,"Unknown option passed.");
    else if (status == 50)
	KMessageBox::information (this,"Can't change directory.");
    else if (status == 51)
	KMessageBox::information (this,"Can't check MD5 sum.");
    else if (status == 52)
	KMessageBox::information (this,"Connection (network) problem.");
    else if (status == 53)
	KMessageBox::information (this,"Can't unlink a file.");
    else if (status == 54)
	KMessageBox::information (this,"MD5 or digital signature verification error.");
    else if (status == 55)
	KMessageBox::information (this,"Error reading file.");
    else if (status == 56)
	KMessageBox::information (this,"Config file error.");
    else if (status == 57)
	KMessageBox::information (this,"Can't create a new file.");
    else if (status == 58)
	KMessageBox::information (this,"Can't read database from remote server.");
    else if (status == 59)
	KMessageBox::information (this,"Mirrors are not fully synchronized (try again later).");
    else if (status == 60)
	KMessageBox::information (this,"Can't get information about clamav user from /etc/passwd.");
    else if (status == 61)
	KMessageBox::information (this,"Can't drop privileges.");
    else
	KMessageBox::information (this,"Warning - Unknown Error!");

	if (errorMessage != "")
		KMessageBox::information (this,errorMessage);
	errorMessage = "";
	if (daemon_box->isChecked())
		status_label->setText(i18n("Update Process Running in Background"));
	else
		status_label->setText( i18n("Ready") );

    //if (status != 0)
	      //matches_label->setText("");

}

void Freshklam::updateMailClient()

{

	config->setGroup("Freshklam");
	
	if (lastDownloadPaths.contains(dir_combo->/*currentText*/url()) == 0) {
	dir_combo->comboBox()->insertItem(dir_combo->/*currentText*/url(), 0);
	lastDownloadPaths.prepend(dir_combo->/*currentText*/url());
		if (lastDownloadPaths.count() > 10) {
			lastDownloadPaths.remove(lastDownloadPaths.fromLast());
			dir_combo->comboBox()->removeItem(dir_combo->comboBox()->count() - 1);
		}
	}else{
		
		lastDownloadPaths.remove(dir_combo->url());
		lastDownloadPaths.prepend(dir_combo->url());
	}
	config->writeEntry("lastDownloadPaths", lastDownloadPaths);
	
	config->writeEntry("ProxyIP", proxyIP->text());
	config->writeEntry("ProxyPort", proxyPort->text());
	config->writeEntry("ProxyUser", proxyUser->text());
	config->writeEntry("ProxyPass", proxyPass->text());
	
	//KMessageBox::information (this,proxyIP->text());
	config->sync();

	
	config = new KConfig("kmailrc");
	config->setGroup("General");
	QVariant nooffilters = config->readEntry("filters");

	int result;	
	int numfilters = nooffilters.toInt();
	nooffilters = numfilters;
	for (int j=0; j != numfilters; j++ ){
		QVariant numb = j;
		QString filtername=QString("Filter #%1").arg(numb.toString());
		config->setGroup(filtername);
		QString binary = config->readEntry("action-args-0");
		if (binary.find("klammail") != -1){
			if (binary.find(dir_combo->url()) == -1){
				QString path = getenv("HOME");
				result = KMessageBox::warningContinueCancelList(0, i18n( "Since you have changed the database location, KlamAV needs to change the parameters used for mail scanning in KMail. The change is displayed below. If you have KMail open you will need to close it now so that the change can take effect. If you want to make the change manually just click 'Cancel'. (NB: You should also kill any instances of clamd running that look like: 'clamd -c %1/klammailxxxxx'so that the database can be reloaded. This will happen automatically in a later version).").arg(path),QString("New filter command: klammail -d %1").arg(dir_combo->url()),i18n( "Update KMail Filters" ),i18n( "Update" ));
				switch (result) {
					case 2 : KMessageBox::information (this,"KMail has not been updated with the new database location.");  break;
					case 5 : 
					config->writeEntry("action-args-0",QString("klammail -d %1").arg(dir_combo->url()));
					config->sync();
					break;

				}
				break;
			}
		}
	}
	KMessageBox::information (this,"Update Complete!");

}
void Freshklam::receivedOutput(KProcess */*proc*/, char *buffer, int buflen)
{
    buf += QCString(buffer, buflen+1);
    processOutput();
}


void Freshklam::slotClear()
{
    finish();
    resultbox->clear();

    status_label->setText(i18n("Ready"));
    matches_label->setText("");
}


void  Freshklam::setDirName(QString dir){
//    dir_combo->setEditText(dir);
    dir_combo->setURL(dir);
}

void Freshklam::writeConf()
{
	KTempFile tf;
	if ( tf.status() != 0 ) {
		tf.close();
		//delete tf;
		KMessageBox::information (this,"KMFilterActionWithCommand: Could not create temp file!");
		return;
	}
	
	
	
	//tf->setAutoDelete(TRUE);
	tempFileName = tf.name();
	
	QTextStream &ts = *(tf.textStream());

	if ( proxyIP->hasAcceptableInput() ) {
		ts << "DatabaseMirror database.clamav.net" << "\n";
		ts << QString("HTTPProxyServer %1").arg(proxyIP->text()) << "\n";
		if ( proxyPort->hasAcceptableInput() ) {
			ts << QString("HTTPProxyPort %1").arg(proxyPort->text()) << "\n";
		}
		if ( proxyUser->hasAcceptableInput() ) {
			ts << QString("HTTPProxyUsername %1").arg(proxyUser->text()) << "\n";
		}
		if ( proxyPass->hasAcceptableInput() ) {
			ts << QString("HTTPProxyPassword %1").arg(proxyPass->text()) << "\n";
		}
	}else{
		return;
	}

	//tf.close();
}

void  Freshklam::handleChecks(){

	config = KGlobal::config();
	config->setGroup("Freshklam");
	if (daemon_box->isChecked()){
		config->writeEntry("AutoUpdate","Yes");
		check_combo->setEnabled(true);
	}else{
		config->writeEntry("AutoUpdate","No");
		check_combo->setEnabled(false);
	}
	config->writeEntry("NoOfUpdates", check_combo->currentText());
	config->sync();
}

bool  Freshklam::killPID(){
	
	QFile inf(pidFileName);
	if (!inf.open(IO_ReadOnly))
		return false;
	QTextStream is(&inf);
	QString line;
	while (!is.eof())
		line = is.readLine();
	inf.close();

	kdDebug() << line << endl;
	if (!(line.isEmpty()))
		kill(line.toInt(), SIGKILL);
	else
		return false;
	if (!(pidFileName.isEmpty()))
		KIO::NetAccess::del(pidFileName,NULL);

	return true;
}

void  Freshklam::disableInputs(){

	daemon_box->setEnabled(false);
	check_combo->setEnabled(false);
	proxyIP->setEnabled(false);
	proxyUser->setEnabled(false);
	proxyPort->setEnabled(false);
	proxyPass->setEnabled(false);
}

void  Freshklam::enableInputs(){

	daemon_box->setEnabled(true);
	check_combo->setEnabled(true);
	proxyIP->setEnabled(true);
	proxyUser->setEnabled(true);
	proxyPort->setEnabled(true);
	proxyPass->setEnabled(true);
}


#include "freshklam.moc"
