/*
 * Copyright (C) 2004 Robert Hogan <robert at roberthogan dot net>
 */

#include "klamscan.h"

#include <config.h>

#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurlrequesterdlg.h>

#include <kedittoolbar.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qregexp.h>
#include <qwhatsthis.h>
#include <kbuttonbox.h>
#include <kfiledialog.h>
#include <kprocess.h>
#include <kprocio.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kwin.h>
#include <kurlrequester.h>
#include <kurlcompletion.h>
#include <kcombobox.h>
#include <kdebug.h>
#include <qlayout.h>
#include <kcmdlineargs.h>
#include <kmessagebox.h>
#include <qtooltip.h>

const char *check_action[] = {
    "Ask me",
    "Quarantine file",
    "Just report",
    0
};


Klamscan::Klamscan(QWidget *parent, const char *name)
    : QWidget(parent, name)
{


	config = KGlobal::config();
	config->setGroup("ScanPaths");
	lastSearchItems = config->readListEntry("LastSearchItems");
	lastSearchPaths = config->readListEntry("LastSearchPaths");
	
	if (lastSearchPaths.isEmpty()){
  		QString path = getenv("HOME");
		lastSearchPaths.append( QString("%1").arg(path));
		config->writeEntry("LastSearchPaths", lastSearchPaths);
		lastSearchPaths = config->readListEntry("LastSearchPaths");
	}
	
	
	
	
	//QGridLayout *layout = new QGridLayout(this, 6, 3, 10, 4);
	QGridLayout *layout = new QGridLayout(this, 6, 3, 10, 4);
	layout->setColStretch(0, 10);
	layout->addColSpacing(1, 10);
	layout->setColStretch(1, 0);
	layout->setColStretch(2, 1);
	layout->addRowSpacing(1, 10);
	layout->setRowStretch(1, 0);
	layout->setRowStretch(2, 10);
	layout->addRowSpacing(4, 10);
	layout->setRowStretch(4, 0);
	
	
	QGridLayout *input_layout = new QGridLayout(2, 2, 4);
	layout->addLayout(input_layout, 0, 0);
	input_layout->setColStretch(0, 0);
	input_layout->setColStretch(1, 20);
	
	
	QLabel *dir_label = new QLabel(i18n("&Directory:"), this);
	dir_label->setFixedSize(dir_label->sizeHint());
	input_layout->addWidget(dir_label, 0, 0, AlignRight | AlignVCenter);
	
	QBoxLayout *dir_layout = new QHBoxLayout(3);
	input_layout->addLayout(dir_layout, 0, 1);
	// anders: KDE is an amazing tool:)
	dir_combo = new KURLRequester( new KComboBox(true, this), this, "dir combo" );
	dir_combo->completionObject()->setMode(KURLCompletion::DirCompletion);
	
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if( args->isSet( "scanthis" ) ) {
	
		for( int i = 0; i < args->count(); i++ ) {
			lastSearchPaths.prepend(args->arg(i));
			if (lastSearchPaths.count() > 10) {
			lastSearchPaths.remove(lastSearchPaths.fromLast());
			dir_combo->comboBox()->removeItem(dir_combo->comboBox()->count() - 1);
			}
			config->writeEntry("LastSearchPaths", lastSearchPaths);
		}
	
	}
	dir_combo->comboBox()->insertStringList(lastSearchPaths);
	dir_combo->setMode( KFile::Directory|KFile::LocalOnly );
	dir_layout->addWidget(dir_combo);
	dir_label->setBuddy(dir_combo);
	
	
	//QBoxLayout *dir_layout2 = new QHBoxLayout(3);
	//input_layout->addLayout(dir_layout2, 0, 1);
	// anders: KDE is an amazing tool:)

	QBoxLayout *dir_layout2 = new QHBoxLayout(3);
	input_layout->addLayout(dir_layout2, 1, 1);


	QLabel *combo_label = new QLabel(i18n("When a virus is found:"), this);
	combo_label->setFixedSize(combo_label->sizeHint());
	dir_layout2->addWidget(combo_label);

	check_combo = new QComboBox(false, this);
	check_combo->insertStrList(check_action);
	check_combo->adjustSize();
	check_combo->setFixedSize(check_combo->size());
	dir_layout2->addWidget(check_combo);

	recursive_box2 = new QCheckBox(i18n("&Scan Quarantine"), this);
	recursive_box2->setMinimumWidth(recursive_box2->sizeHint().width());
	recursive_box2->setChecked(false);
	dir_layout2->addSpacing(10);
	dir_layout2->addWidget(recursive_box2);
	QToolTip::add( recursive_box2, "Scan the Quarantine if it is found in or under the path." );
	dir_combo->setMinimumWidth(recursive_box2->sizeHint().width());
	
	recursive_box = new QCheckBox(i18n("Scan &Sub-Dirs"), this);
	recursive_box->setMinimumWidth(recursive_box->sizeHint().width());
	recursive_box->setChecked(true);
	dir_layout2->addSpacing(10);
	dir_layout2->addWidget(recursive_box);
	QToolTip::add( recursive_box, "Scan all directories under the specified path." );

	
	KButtonBox *actionbox = new KButtonBox(this, Qt::Vertical);
	layout->addWidget(actionbox, 0, 2);
	actionbox->addStretch();
	search_button = actionbox->addButton(i18n("&Scan"));
	search_button->setDefault(true);
	cancel_button = actionbox->addButton(i18n("Cancel"));
	cancel_button->setEnabled(false);
	actionbox->addStretch();
	actionbox->layout();
	
	resultbox = new QListBox(this);
	QFontMetrics rb_fm(resultbox->fontMetrics());
	resultbox->setMinimumSize(rb_fm.width("0")*55,
				rb_fm.lineSpacing()*15);
	layout->addMultiCellWidget(resultbox, 2, 2, 0, 2);
	
	QFrame *status_frame = new QFrame(this);
	status_frame->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	QBoxLayout *status_layout = new QHBoxLayout(status_frame, 2);
	

	status_label = new QLabel("", status_frame);
	status_layout->addWidget(status_label, 10);
	
	//matches_label = new QLabel(status_frame);
	//QFontMetrics ml_fm(matches_label->fontMetrics());
	//matches_label->setFixedWidth(ml_fm.width(i18n("9999 viruseses/errors found")));
	//matches_label->setFixedHeight(ml_fm.lineSpacing());
	//status_layout->addWidget(matches_label, 0);
	
	
	status_layout->activate();
	status_frame->adjustSize();
	status_frame->setMinimumSize(status_frame->size());
	layout->addMultiCellWidget(status_frame, 3, 3, 0, 2);

	QFrame *status2_frame = new QFrame(this);
	status2_frame->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	QBoxLayout *status2_layout = new QHBoxLayout(status2_frame, 2);
	
	status2_label = new QLabel(i18n("Ready"), status2_frame);
	status2_layout->addWidget(status2_label, 10);

	matches2_label = new QLabel(status2_frame);
	QFontMetrics ml_fm2(matches2_label->fontMetrics());
	matches2_label->setFixedWidth(ml_fm2.width(i18n("9999 viruseses/errors found")));
	matches2_label->setFixedHeight(ml_fm2.lineSpacing());
	status2_layout->addWidget(matches2_label, 0);
	
	status2_layout->activate();
	status2_frame->adjustSize();
	status2_frame->setMinimumSize(status2_frame->size());
	layout->addMultiCellWidget(status2_frame, 4, 4, 0, 2);

		
	layout->activate();

	
	
	connect( search_button, SIGNAL(clicked()),
		SLOT(slotSearch()) );
	connect( cancel_button, SIGNAL(clicked()),
		SLOT(slotCancel()) );
	
	
	
	if( args->isSet( "scanthis" ) ) {
	slotSearch();
	}


}

Klamscan::~Klamscan()
{
}



void Klamscan::processOutput()
{
    int pos;
    QString item2;


	buf.replace("//","/"); // don't know why they're getting two slashes
	for (int j=0; buf.section('\n',j,j) != ""; j++ ){
			if ((pos = buf.section('\n',j,j).find("FOUND")) != -1){
				item2 = (buf.section('\n',j,j)).stripWhiteSpace();
				int fnameStartPoint = 0;
				int fnameEndPoint = item2.findRev(":");
				QString tmpFName = item2.mid(fnameStartPoint,(fnameEndPoint - fnameStartPoint));
				QString tmpVirusName = item2.mid((fnameEndPoint+1),(item2.length() - (fnameEndPoint+1)));
				
				kdDebug() << "tmpFName " << tmpFName << endl;
				kdDebug() << "tmpVirusName " << tmpVirusName << endl;
				if ((tmpVirusName.find("FOUND")) != -1){
					resultbox->insertItem(buf.section('\n',j,j));
					QuarantineList.append(buf.section('\n',j,j));
				}
			}
			if ((pos = buf.section('\n',j,j).find("ERROR:")) != -1){
				resultbox->insertItem(buf.section('\n',j,j));
				errorsEncountered = TRUE;	
			}
			if ((pos = buf.section('\n',j,j).find("OK")) != -1){
				status_label->setText(i18n("Scanning... %1").arg(buf.section('\n',j,j)));
				
			}
		}

	buf = "";

	QString str;
	str.setNum(resultbox->count());

	str += i18n(" viruseses/errors found");
	matches2_label->setText(str);
}


void Klamscan::slotSearch()
{
	slotClear();
	QuarantineList.clear();
	search_button->setEnabled(false);
	cancel_button->setEnabled(true);
	
	errorsEncountered = FALSE;
	status2_label->setText(i18n("Scan in Progress..."));

	config = KGlobal::config();
	config->setGroup("Freshklam");
	QStringList lastDownloadPaths = config->readListEntry("lastDownloadPaths");
	QString db;
	for (QStringList::Iterator ita = lastDownloadPaths.begin(); ita == lastDownloadPaths.begin() ; ita++){
			db = *ita;
	}
	QString filepattern;
	
	if (!(db.isEmpty()))
		filepattern = QString(" -d %1 ").arg(db);
	
	if (recursive_box2->isChecked()){
		config->setGroup("Kuarantine");
		QStringList lastQuarLocations = config->readListEntry("KuarantineLocations");
		QString quarloc;
		for (QStringList::Iterator ita = lastQuarLocations.begin(); ita == lastQuarLocations.begin() ; ita++){
				quarloc = *ita;
		}
		filepattern += QString(" --exclude=%1 ").arg(quarloc);
	}

	
	if (recursive_box->isChecked())
		filepattern += " -r ";
	filepattern += " ";
	filepattern +=  KProcess::quote(dir_combo->/*currentText*/url());
	
	childproc = new KShellProcess();
	*childproc << "clamscan --stdout";
	*childproc << filepattern;
	
	connect( childproc, SIGNAL(processExited(KProcess *)),
		SLOT(childExited()) );
	connect( childproc, SIGNAL(receivedStdout(KProcess *, char *, int)),
		SLOT(receivedOutput(KProcess *, char *, int)) );
	// actually it should be checked whether the process was started succesfully
	/*bool success=*/childproc->start(KProcess::NotifyOnExit, KProcess::Stdout);
}


void Klamscan::finish()
{
    search_button->setEnabled(true);
    cancel_button->setEnabled(false);

    buf += '\n';
    processOutput();
    delete childproc;
    childproc = 0;

    config->setGroup("ScanPaths");
    if (lastSearchPaths.contains(dir_combo->/*currentText*/url()) == 0) {
        dir_combo->comboBox()->insertItem(dir_combo->/*currentText*/url(), 0);
        lastSearchPaths.prepend(dir_combo->/*currentText*/url());
        if (lastSearchPaths.count() > 10) {
            lastSearchPaths.remove(lastSearchPaths.fromLast());
            dir_combo->comboBox()->removeItem(dir_combo->comboBox()->count() - 1);
        }
    }else{
    	lastSearchPaths.remove(dir_combo->/*currentText*/url());
	lastSearchPaths.prepend(dir_combo->/*currentText*/url());
    }
    config->writeEntry("LastSearchPaths", lastSearchPaths);
    config->sync();

    status_label->setText("");

}


void Klamscan::slotCancel()
{
    finish();

	status2_label->setText(i18n("Cancelled"));

}


void Klamscan::childExited()
{
    int status = childproc->exitStatus();

    finish();
    int result;
    
    status2_label->setText( i18n("Scan Complete") );

    if (status == 0)
    	if (!(errorsEncountered))
		KMessageBox::information (this,"Scan Complete - No Viruses Found!");
	else
    		KMessageBox::information (this,"Scan Complete - No Viruses Found But Some Errors Encountered!");

    else if(status == 1){
    	switch (check_combo->currentItem()) {
		case 0:
			result = KMessageBox::warningContinueCancelList(0, i18n( "I'm going to quarantine this lot, you can restore them later if you want. If you don't want to quarantine, just press cancel."),QuarantineList,i18n( "Quarantine Infected Files" ),i18n( "Quarantine" ));
			switch (result) {
				case 2 : KMessageBox::information (this,"Infected Items have not been quarantined.");  break;
				case 5 : Quarantine(); break;
			}
			break;
		case 1:
			Quarantine(); break;
		default:
			KMessageBox::information (this,"Scan Complete - Viruses Found!");break;
	}
    }else{
    	KMessageBox::information (this,"Unspecified Error!");
    }

    status2_label->setText( i18n("Ready") );

    //if (status != 0)
	      //matches_label->setText("");

}


void Klamscan::receivedOutput(KProcess */*proc*/, char *buffer, int buflen)
{
    buf += QCString(buffer, buflen+1);
    processOutput();
}


void Klamscan::slotClear()
{
    //finish();
    resultbox->clear();

    status2_label->setText(i18n("Ready"));
    matches2_label->setText("");
}


void  Klamscan::setDirName(QString dir){
//    dir_combo->setEditText(dir);
    dir_combo->setURL(dir);
}

void  Klamscan::Quarantine(){
	
	config = KGlobal::config();
	config->setGroup("Kuarantine");
	//lastQuarItems = config->readListEntry("KuarantineItems");
	QStringList lastQuarLocations = config->readListEntry("KuarantineLocations");
	
	QString quarloc;
	for (QStringList::Iterator it = lastQuarLocations.begin(); it == lastQuarLocations.begin() ; it++){
			quarloc = *it;
	}
		
	//Get Items for this location
	
	lastQuarItems = config->readListEntry(QString("Items %1").arg(quarloc));
	
	
	for (QStringList::Iterator it = QuarantineList.begin(); it != QuarantineList.end(); it++ ){
		if (lastQuarItems.contains(*it) != 0) {
			lastQuarItems.remove(*it);
		}
		
		QStringList tokens = QStringList::split ( ":", *it, FALSE );
		QString fname = tokens.first();
		
		if (!(fname.isEmpty())){
			QStringList itemName = QStringList::split ( "FOUND", *it, FALSE );
			tokens = QStringList::split ( "/", fname, FALSE );
			QString qname = tokens.last();
			qname.prepend("/");
			qname.prepend(quarloc);
			/*
			QString suCommand=QString("mv -f '%1' '%2'").arg(fname).arg(qname);
			KProcIO *proc = new KProcIO();
			proc->setUseShell(TRUE);
			*proc<<suCommand;
			
			proc->start(KProcIO::NotifyOnExit);
			*/
			if (KIO::NetAccess::file_move(fname,qname)){
				if (lastQuarItems.contains(itemName.first()))
					lastQuarItems.remove(itemName.first());
				lastQuarItems.prepend(itemName.first());
			}else{
				KMessageBox::information (this,i18n("<p>There was a problem quarantining <b>%1</b>. Check your diskspace and the permissions on your quarantine location. </p>").arg(fname));
			}

			
		}
	}
	config->writeEntry(QString("Items %1").arg(quarloc), lastQuarItems);
	config->sync();




}



#include "klamscan.moc"
