/*
 * Copyright (C) 2004 Robert Hogan <robert at roberthogan dot net>
 */
#include <config.h>

#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#include "kuarantine.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kkeydialog.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurlrequesterdlg.h>

#include <kedittoolbar.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qregexp.h>
#include <qwhatsthis.h>
#include <kbuttonbox.h>
#include <kfiledialog.h>
#include <kprocess.h>
#include <kprocio.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kwin.h>
#include <kurlrequester.h>
#include <kurlcompletion.h>
#include <kcombobox.h>
#include <kdebug.h>
#include <qlayout.h>
#include <kcmdlineargs.h>
#include <kmessagebox.h>
#include <klocale.h>


Kuarantine::Kuarantine(QWidget *parent, const char *name)
    : QWidget(parent, name)
{

	config = KGlobal::config();
	config->setGroup("Kuarantine");
	lastQuarLocations = config->readListEntry("KuarantineLocations");

	
	if (lastQuarLocations.isEmpty()){
		createQuarDir();
 	}else{
		for (QStringList::Iterator it = lastQuarLocations.begin(); it == lastQuarLocations.begin(); it++ )
		quarloc = *it;
		if (!(KIO::NetAccess::exists(quarloc,TRUE,NULL))){
			KMessageBox::information (this,QString("Your current quarantine location ('%1') no longer exists. I'm going to attempt to create a new one.").arg(quarloc));
			createQuarDir();
		}
	}
		

		
	//Get Items for this location

	lastQuarItems = config->readListEntry(QString("Items %1").arg(quarloc));

	if (lastQuarItems.isEmpty()){
		config->writeEntry(QString("Items %1").arg(quarloc), "");
		lastQuarItems = config->readListEntry(QString("Items %1").arg(quarloc));
	}
	
	//Get History for this location

	lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));

	if (lastHistoryItems.isEmpty()){
		config->writeEntry(QString("History %1").arg(quarloc), "");
		lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));
	}
	
	//Create GUI	

	QVBoxLayout *vbox = new QVBoxLayout(this, KDialog::marginHint(),
			KDialog::spacingHint(), "vbox");

	QGroupBox *group = new QGroupBox(i18n("Quarantine Location"), this);
	vbox->addWidget(group);

	QGridLayout *layout = new QGridLayout( group, 9, 3, KDialog::spacingHint(),
			KDialog::spacingHint(), "layout");
	layout->addRowSpacing(0, group->fontMetrics().height());
	layout->setColStretch(0, 1);
	layout->setColStretch(1, 1);

	QWidget *hlp = new QWidget( group );
	layout->addMultiCellWidget(hlp, 1,1, 0,2);
	QHBoxLayout *dir_layout = new QHBoxLayout(hlp, KDialog::spacingHint() );

	QLabel *dir_label = new QLabel(i18n("&Directory:"), hlp);
	dir_label->setFixedSize(dir_label->sizeHint());
	dir_layout->addWidget(dir_label);


	// anders: KDE is an amazing tool:)
	dir_combo = new KURLRequester( new KComboBox(true, hlp), hlp, "dir combo" );
	dir_combo->completionObject()->setMode(KURLCompletion::DirCompletion);

	dir_combo->comboBox()->insertStringList(lastQuarLocations);
	dir_combo->setMode( KFile::Directory|KFile::LocalOnly );
	dir_layout->addWidget(dir_combo);
	dir_label->setBuddy(dir_combo);
	
	connect( dir_combo, SIGNAL(urlSelected(const QString &)),
		SLOT(updateLastQuarLocations()) );
	connect( dir_combo, SIGNAL(textChanged(const QString &)),
		SLOT(updateLastQuarLocations()) );



	QGroupBox *group2 = new QGroupBox(i18n("Contents Of Quarantine"), this);
	vbox->addWidget(group2);

	QGridLayout *layout2 = new QGridLayout( group2, 9, 3, KDialog::spacingHint(),
			KDialog::spacingHint(), "layout2");
	layout2->addRowSpacing(0, group2->fontMetrics().height());
	layout2->setColStretch(0, 1);
	layout2->setColStretch(1, 1);

	QWidget *hlp2 = new QWidget( group2 );
	layout2->addMultiCellWidget(hlp2, 1,1, 0,2);
	QHBoxLayout *dir_layout2 = new QHBoxLayout(hlp2, KDialog::spacingHint() );


	currentbox = new QListBox(hlp2);
	QFontMetrics rb_fm(currentbox->fontMetrics());
	currentbox->setMinimumSize(rb_fm.width("0")*25,
				rb_fm.lineSpacing()*8);
	dir_layout2->addWidget(currentbox);



	refresh();

	KButtonBox *actionbox = new KButtonBox(hlp2, Qt::Vertical);
	actionbox->addStretch();
	refresh_button = actionbox->addButton(i18n("&Refresh"));
	refresh_button->setDefault(true);
	search_button = actionbox->addButton(i18n("&Restore"));
	search_button->setDefault(true);
	cancel_button = actionbox->addButton(i18n("&Delete"));
	actionbox->addStretch();
	actionbox->layout();

	dir_layout2->addWidget(actionbox);


	connect( refresh_button, SIGNAL(clicked()),
		SLOT(refresh()) );

	connect( search_button, SIGNAL(clicked()),
		SLOT(slotSearch()) );
	connect( cancel_button, SIGNAL(clicked()),
		SLOT(slotCancel()) );



	QGroupBox *group3 = new QGroupBox(i18n("Quarantine History"), this);
	vbox->addWidget(group3);

	QGridLayout *layout3 = new QGridLayout( group3, 9, 2, KDialog::spacingHint(),
			KDialog::spacingHint(), "layout3");
	layout3->addRowSpacing(0, group3->fontMetrics().height());
	layout3->setColStretch(0, 1);
	layout3->setColStretch(1, 1);

	QWidget *hlp3 = new QWidget( group3 );
	layout3->addMultiCellWidget(hlp3, 1,1, 0,2);
	QHBoxLayout *dir_layout3 = new QHBoxLayout(hlp3, KDialog::spacingHint() );


	historybox = new QListBox(hlp3);
	QFontMetrics rb_fm3(historybox->fontMetrics());
	historybox->setMinimumSize(rb_fm3.width("0")*25,
				rb_fm3.lineSpacing()*8);
	dir_layout3->addWidget(historybox);

	KButtonBox *actionbox3 = new KButtonBox(hlp3, Qt::Vertical);
	actionbox3->addStretch();
	search_button3 = actionbox3->addButton(i18n("&Clear All"));
	search_button3->setDefault(true);
	cancel_button3 = actionbox3->addButton(i18n("&Delete"));
	cancel_button3->setEnabled(true);
	actionbox3->addStretch();
	actionbox3->layout();

	dir_layout3->addWidget(actionbox3);

	refreshhistory();

	connect( search_button3, SIGNAL(clicked()),
		SLOT(slotClearHistory()) );
	connect( cancel_button3, SIGNAL(clicked()),
		SLOT(slotDeleteHistory()) );





}

Kuarantine::~Kuarantine()
{
}

void Kuarantine::refresh()
{
	QStringList obsoleteQuarItems;
	config = KGlobal::config();
	config->setGroup("Kuarantine");
	lastQuarItems = config->readListEntry(QString("Items %1").arg(quarloc));
	
	if (!(KIO::NetAccess::exists(quarloc,TRUE,NULL))){
		KMessageBox::information (this,QString("Your current quarantine location ('%1') no longer exists. I'm going to attempt to create a new one.").arg(quarloc));
		createQuarDir();
	}else{

		currentbox->clear();
		for (QStringList::Iterator it = lastQuarItems.begin(); it != lastQuarItems.end(); it++ ){
				int fnameEndPoint = QString(*it).findRev(":");
				QString fname = QString(*it).mid(0,fnameEndPoint);
	
				QStringList tokens = QStringList::split ( "/", fname, FALSE );
				QString qname = tokens.last();
				qname.prepend("/");
				qname.prepend(quarloc);
				kdDebug() << qname << endl;
				if (KIO::NetAccess::exists(qname,TRUE,NULL)){
					currentbox->insertItem(*it);
				}else{
					kdDebug() << qname << "doesna exist" << endl;
					obsoleteQuarItems.append(*it);
				}
		}
	
		for (QStringList::Iterator it = obsoleteQuarItems.begin(); it != obsoleteQuarItems.end(); it++ )
			lastQuarItems.remove(*it);
		
		config->writeEntry(QString("Items %1").arg(quarloc), lastQuarItems);
	}
	config->sync();
	
}

void Kuarantine::refreshhistory()
{
    config = KGlobal::config();
    config->setGroup("Kuarantine");
    lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));

	historybox->clear();
	for (QStringList::Iterator it = lastHistoryItems.begin(); it != lastHistoryItems.end(); it++ )
		    	historybox->insertItem(*it);
}

void Kuarantine::load()
{
    QString target;
    // the below code is what you should normally do.  in this
    // example case, we want the url to our own.  you probably
    // want to use this code instead for your app

    	currentbox->clear();
	for (QStringList::Iterator it = lastQuarItems.begin(); it != lastQuarItems.end(); it++ )
		    	currentbox->insertItem(*it);

}



void Kuarantine::slotSearch()
{
	if (!(currentbox->selectedItem()))
		return;

	QListBoxItem* tItem = currentbox->selectedItem();
	config = KGlobal::config();
	config->setGroup("Kuarantine");
	lastQuarItems = config->readListEntry(QString("Items %1").arg(quarloc));
	lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));




	int fnameEndPoint = tItem->text().findRev(":");
	QString fname = tItem->text().mid(0,fnameEndPoint);

	QStringList tokens = QStringList::split ( "/", fname, FALSE );
	QString qname = tokens.last();
	qname.prepend("/");
	qname.prepend(quarloc);
         /*
	 QString suCommand=QString("mv -f '%1' '%2'").arg(qname).arg(fname);
	childproc = new KProcIO();
	childproc->setUseShell(TRUE);
	*childproc<<suCommand;


	connect( childproc, SIGNAL(readReady(KProcIO *)),
		SLOT(childproc->readln(QString *)) );
	childproc->start(KProcIO::NotifyOnExit);
*/
//	KIO::NetAccess::file_move(qname,fname);
//	if (childproc->exitStatus() == 0){
	if (KIO::NetAccess::file_move(qname,fname)){
		lastQuarItems.remove(tItem->text());
		
		currentbox->removeItem(currentbox->currentItem());
		
		config->writeEntry(QString("Items %1").arg(quarloc), lastQuarItems);
		config->sync();
	}else{
		KMessageBox::information (this,"There was a problem restoring the file. Check your diskspace and the permissions on the folder the file is being restored to. ");
	}
}


void Kuarantine::updateLastQuarLocations()
{


	config = KGlobal::config();
	config->setGroup("Kuarantine");
	lastQuarLocations = config->readListEntry("KuarantineLocations");
	
	if (lastQuarLocations.contains(dir_combo->url()) == 0) {
		dir_combo->comboBox()->insertItem(dir_combo->url(), 0);
		lastQuarLocations.prepend(dir_combo->url());
		if (lastQuarLocations.count() > 10) {
			lastQuarLocations.remove(lastQuarLocations.fromLast());
			dir_combo->comboBox()->removeItem(dir_combo->comboBox()->count() - 1);
		}
	}else{
		
		lastQuarLocations.remove(dir_combo->url());
		lastQuarLocations.prepend(dir_combo->url());
	}
	config->writeEntry("KuarantineLocations", lastQuarLocations);
	config->sync();
	quarloc = dir_combo->url();
	
	refresh();
	refreshhistory();
}


void Kuarantine::slotCancel()
{

	if (!(currentbox->selectedItem()))
		return;

	QListBoxItem* tItem = currentbox->selectedItem();
    config = KGlobal::config();
    config->setGroup("Kuarantine");
    lastQuarItems = config->readListEntry(QString("Items %1").arg(quarloc));
    lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));

    QString QuarLocation = dir_combo->url();



	QStringList tokens = QStringList::split ( ":", tItem->text(), FALSE );
	QString fname = tokens.first();

	tokens = QStringList::split ( "/", fname, FALSE );
	QString qname = tokens.last();
	qname.prepend("/");
	qname.prepend(QuarLocation);
         /*
	 QString suCommand=QString("rm -f '%1'").arg(qname);
	childproc = new KProcIO();
	childproc->setUseShell(TRUE);
	*childproc<<suCommand;
	
	connect( childproc, SIGNAL(readReady(KProcIO *)),
		SLOT(childproc->readln(QString *)) );
	childproc->start(KProcIO::NotifyOnExit);
	*/
	if (KIO::NetAccess::del(qname,NULL)){

		lastQuarItems.remove(tItem->text());
		lastHistoryItems.prepend(tItem->text());
	
		historybox->insertItem(tItem->text());
		currentbox->removeItem(currentbox->currentItem());
	
		config->writeEntry(QString("Items %1").arg(quarloc), lastQuarItems);
		config->writeEntry(QString("History %1").arg(quarloc), lastHistoryItems);
		config->sync();
	}else{
		KMessageBox::information (this,"There was a problem deleting the file. Is there a problem with the permissions on the quarantine folder? ");
	}


}





void Kuarantine::slotClearHistory()
{

    historybox->clear();
    config = KGlobal::config();
    config->setGroup("Kuarantine");
    lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));

    if (lastHistoryItems.isEmpty())
    	return;

	lastHistoryItems.clear();

	 config->writeEntry(QString("History %1").arg(quarloc), lastHistoryItems);
	 config->sync();



}

void Kuarantine::slotDeleteHistory()
{

	if (!(historybox->selectedItem()))
		return;

	QListBoxItem* tItem = historybox->selectedItem();

    config->setGroup("Kuarantine");
    lastHistoryItems = config->readListEntry(QString("History %1").arg(quarloc));

    lastHistoryItems.remove(tItem->text());

    historybox->removeItem(historybox->currentItem());
	 config->writeEntry(QString("History %1").arg(quarloc), lastHistoryItems);
	 config->sync();

}


void  Kuarantine::setDirName(QString dir){
//    dir_combo->setEditText(dir);
    dir_combo->setURL(dir);
}

void  Kuarantine::createQuarDir(){

	QString path = getenv("HOME");
	bool ok = true;
	// directory exist?
	path += "/.klamav";
	QDir klamavdir(path);
	if (!klamavdir.exists() && !klamavdir.mkdir(path))
		ok = false;

	path += "/quarantine";
	if (ok)
	{
		QDir klamavqdir(path);
		if (!klamavqdir.exists() && !klamavqdir.mkdir(path))
			ok = false;
	}


	lastQuarLocations.append( QString("%1").arg(path));
	config->writeEntry("KuarantineLocations", lastQuarLocations);
	lastQuarLocations = config->readListEntry("KuarantineLocations");
	KMessageBox::information (this,QString(ok ? "Your default quarantine location has been set up as '%1'. You can change this to something else if you want to." : "I cannot create the directory '%1' for you. Something is wrong with your HOME or klamav directory. You have to adjust your quarantine directory by your self.").arg(path));

}

#include "kuarantine.moc"
